snakeImgs = {["down"]=love.graphics.newImage("snakehead_down.png"),["up"]=love.graphics.newImage("snakehead_up.png"),["right"]=love.graphics.newImage("snakehead_right.png"),["left"]=love.graphics.newImage("snakehead_left.png")}

blockSize = 5

lvlWidth = love.graphics.getWidth()/blockSize
lvlHeight = love.graphics.getHeight()/blockSize

tickTimer = 0
tickTimerMax = 0.1

directions = {"up","right","down","left"}

pieces = {}

particles = {}
function updateParticles()
  for i,p in pairs(particles) do
    p.x = p.x + p.vx
    p.y = p.y + p.vy
    p.timer = p.timer + 0.1
    if p.timer >= p.lifetime then table.remove(particles,i) end
  end
end
function drawParticles()
  for i,p in pairs(particles) do
    love.graphics.setColor(p.color[1],p.color[2],p.color[3],255-((255/p.lifetime)*p.timer))
    love.graphics.circle("fill",p.x,p.y,p.radius)
    love.graphics.setColor(255,255,255,255)
  end
end
function addParticle(x,y,color,vx,vy,radius,lifetime)
  table.insert(particles,{x=x,y=y,color=color,vx=vx,vy=vy,radius=radius,lifetime=lifetime,timer=0})
end

math.randomseed(os.time())

particlesOnCrash = 20

gameState = "fail"

function restart()
  defSnake1 = newSnake(10,10,{255,0,0},"down",'a','d','w','s')
  defSnake2 = newSnake(90,10,{0,255,0},"down",'left','right','up','down')
  snakes = {defSnake1,defSnake2}
  pieces = {}
  countTimer = 5
  gameState = "getready"
end

function indexOf(T,f,ind)
  if ind == nil then ind = 0 end
  local found = false
  local tInd = 0
  for i,thing in ipairs(T) do
    if thing == f and i > ind and not found then
      tInd = i
      found = true
    end
  end
  return tInd
end

function newSnake(x,y,color,direction,leftKey,rightKey,upKey,downKey,id)
  local snake = {}
  snake.x = x
  snake.y = y
  snake.color = color
  snake.direction = direction
  snake.img = snakeImgs[snake.direction]
  snake.leftKey = leftKey
  snake.rightKey = rightKey
  snake.upKey = upKey
  snake.downKey = downKey
  snake.leftDir = directions[indexOf(directions,snake.direction) + 1]
  snake.rightDir = directions[indexOf(directions,snake.direction) - 1]
  snake.id = id
  if snake.leftDir == nil and snake.rightDir ~= nil then snake.leftDir = "up" end
  if snake.rightDir == nil and snake.leftDir ~= nil then snake.rightDir = "left" end
  return snake
end

function updateSnake(snake)
  table.insert(pieces,{x=snake.x,y=snake.y,color=snake.color,id=snake.id})
  snake.img = snakeImgs[snake.direction]
  if snake.direction == "down" then snake.y = snake.y + 1
  elseif snake.direction == "up" then snake.y = snake.y - 1
  elseif snake.direction == "right" then snake.x = snake.x + 1
  elseif snake.direction == "left" then snake.x = snake.x - 1 end
  if snake.x <= 0 and snake.direction == "left" then snake.x = lvlWidth end
  if snake.x > lvlWidth and snake.direction == "right" then snake.x = 1 end
  if snake.y <= 0 and snake.direction == "up" then snake.y = lvlHeight end
  if snake.y > lvlHeight and snake.direction == "down" then snake.y = 1 end
  return snake
end

defSnake1 = newSnake(10,10,{255,0,0},"down",'a','d','w','s',1)
defSnake2 = newSnake(90,10,{0,255,0},"down",'left','right','up','down',2)
snakes = {defSnake1,defSnake2}
scores = {0,0}

countTimer = 5

function love.update(dt)
  local tm = dt * 100
  if gameState == "game" then
    for _,snake in pairs(snakes) do
      if love.keyboard.isDown(snake.leftKey) and snake.direction ~= "right" then
        snake.direction = "left"
      elseif love.keyboard.isDown(snake.rightKey) and snake.direction ~= "left" then
        snake.direction = "right"
      elseif love.keyboard.isDown(snake.upKey) and snake.direction ~= "down" then
        snake.direction = "up"
      elseif love.keyboard.isDown(snake.downKey) and snake.direction ~= "up" then
        snake.direction = "down"
      end
    end
    if tickTimer < tickTimerMax then tickTimer = tickTimer + 0.1 * tm else
      tickTimer = 0
      onTick()
    end
  elseif gameState == "getready" then
    countTimer = countTimer - 0.01 * tm
    if countTimer <= 0 then gameState = "game" end
  elseif gameState == "fail" then
    if love.keyboard.isDown('return') then restart() end
  end
  updateParticles()
end

function onTick()
  for _,snake in pairs(snakes) do
    snake = updateSnake(snake)
  end
  for i,snake in pairs(snakes) do
    for _,piece in pairs(pieces) do
      if snake.x == piece.x and snake.y == piece.y then
        gameState = "fail"
        for j=1,particlesOnCrash do
          addParticle(snake.x*blockSize,snake.y*blockSize,snake.color,math.random(-3,3),math.random(-3,3),math.random(3,5),math.random(3,5))
        end
        --restart()
        scores[3-i] = scores[3-i] + 1
      end
    end
    for j,ssnake in pairs(snakes) do
      if ssnake.x == snake.x and snake.y == ssnake.y and j ~= i then
        gameState = "fail"
        for j=1,particlesOnCrash do
          addParticle(snake.x*blockSize,snake.y*blockSize,snake.color,math.random(-3,3),math.random(-3,3),math.random(3,5),math.random(3,5))
        end
        for j=1,particlesOnCrash do
          addParticle(ssnake.x*blockSize,ssnake.y*blockSize,ssnake.color,math.random(-3,3),math.random(-3,3),math.random(3,5),math.random(3,5))
        end
        --restart()
      end
    end
  end
end

function love.draw()
  for i,snake in pairs(snakes) do
    local toprint = ""
    for j=1,i-1 do toprint = toprint .. "\n" end
    toprint = toprint .. scores[i]
    love.graphics.setColor(snake.color[1],snake.color[2],snake.color[3])
    love.graphics.draw(snake.img,(snake.x-1)*blockSize,(snake.y-1)*blockSize)
    love.graphics.scale(2,2)
    love.graphics.print(toprint)
    love.graphics.reset()
  end
  for i,piece in ipairs(pieces) do
    love.graphics.setColor(piece.color[1],piece.color[2],piece.color[3])
    love.graphics.rectangle("fill",(piece.x-1)*blockSize,(piece.y-1)*blockSize,blockSize,blockSize)
  end
  if gameState == "getready" then
    love.graphics.scale(10,10)
    love.graphics.print(math.ceil(countTimer),20,20)
  elseif gameState == "fail" then
    love.graphics.setColor(255,255,255)
    love.graphics.print("Press ENTER to continue",love.graphics.getWidth()/2,love.graphics.getHeight()/2)
  end
  drawParticles()
end